

#include "parse_tdb.h"


/* ***********************************************************
*********************************************************** */
radar_type::radar_type() {

   links = NULL;
   semcode = NULL;
   receiver_gain = transmitter_gain = NULL;
   sensor_type = SENSOR_OTHER;
}


/* ***********************************************************
  This method extract capability names for a radar platform

   FILE *infile,                       // input file
   char *token,                        // last processed token
   int *errorflag                      // error flag
   dbl_llist_manager *clist,           // list of extracted capabilities
   dbl_llist_manager *susceptibility,  // list of extracted susceptibilities
   int type                            // receiver/transmitter flag
*********************************************************** */
void radar_type::extract_tdbdat(FILE *infile, char *token, int *errorflag, dbl_llist_manager *clist, int type, dbl_llist_manager *semcode_table) {
      
   capability_type *ptr;
   capability_list_type *ctr;
   
   *errorflag = get_sweg_token(infile, token);	// radar id
   id = atoi(token);
   
   *errorflag = get_sweg_token(infile, token);	// radar name
   name.stringcpy(token);
   
   if (semcode_table->head)
      semcode = ((name2sem *)semcode_table->head)->find(token);

   *errorflag = get_sweg_token(infile, token);

   while (*errorflag) {
      if (!strcmp(token, TOKEN_CAPABILITY)) {
         get_sweg_token(infile, token);

         ptr = clist->head ? ((capability_type *)clist->head)->find(token) : NULL;

         if (!ptr) {
            clist->insert(ptr = new capability_type, NULL);
            ptr->name.stringcpy(token);
         }

         capability_man.insert(ctr = new capability_list_type, NULL);
         ctr->item = ptr;

         ptr->typeflag |= type;

         *errorflag = get_sweg_token(infile, token);
         continue;
      }
      
      if (!strcmp(token, TOKEN_LINKAGES)  ||
          !strcmp(token, TOKEN_PLATFORM)  ||
          !strcmp(token, TOKEN_ELEMENT)   ||
          !strcmp(token, TOKEN_THINKER)   ||
          !strcmp(token, TOKEN_MOVER)     ||
          !strcmp(token, TOKEN_DISRUPTOR) ||
          !strcmp(token, TOKEN_WEAPON)    ||
          !strcmp(token, TOKEN_COMM_XMTR) ||
          !strcmp(token, TOKEN_COMM_RCVR) ||
          !strcmp(token, TOKEN_SNR_XMTR)  ||
          !strcmp(token, TOKEN_SNR_RCVR)  ||
          !strcmp(token, TOKEN_SUSCEPTIBILITY)  ||
          !strcmp(token, TOKEN_PLAYER_STRUCTURE))
         break;

      *errorflag = get_sweg_token(infile, token);
   }

}


/* ***********************************************************
   This method searches for a specific radar using ids specified
   by the LINKAGE statement

   int x // linkage id
*********************************************************** */
radar_type *radar_type::find_radar(int x) {

   if (x == id)
      return this;
      
   return next ? ((radar_type *)next)->find_radar(x) : (radar_type *)NULL;
}


/* **********************************************************************
   This method performs preprocesses much of the calculations used by
   calc_detection_range()

   dbl_llist_manager *linkage_man   // list of transmitter/receiver linkages
********************************************************************** */
void radar_type::init_receiver(dbl_llist_manager *linkage_man) {

   capability_list_type *radar;
   capability_type *rec = NULL;
   capability_type *xmit = NULL;
   double loss, range4;
   double transmitter_loss;
   double receiver_loss;
   double operating_loss;
   double gain;

   for (radar=(capability_list_type *)capability_man.head; radar; radar=(capability_list_type *)radar->next)
      if ((radar->item->typeflag & RADAR_RECEIVER) && (radar->item->sensor_type == SENSOR_EM || radar->item->sensor_type == SENSOR_IR))
         break;

   if (!radar)
      return;

   sensor_type = radar->item->sensor_type;
   rec = radar->item;

   // find xmitter through links
   for (radar=NULL, links = (linkage_type *)linkage_man->head; links; links=(linkage_type *)links->next) {
      if (id == links->first) {
         radar = (capability_list_type *)links->secondptr->capability_man.head;
         break;
      }

      if (id == links->second) {
         radar = (capability_list_type *)links->firstptr->capability_man.head;
         break;
      }

   }

   for (; radar && !(radar->item->typeflag & RADAR_TRANSMITTER); radar=(capability_list_type *)radar->next);

   receiver_freq = rec->receiver_freq;
   wavelength2 = SPEED_LIGHT/receiver_freq;
   wavelength2 *= wavelength2;
   maxrange = rec->one_m2_detect_range;

   range4 = rec->one_m2_detect_range * rec->one_m2_detect_range;

   // look up transmitter internal loss, output power, etc
   if (radar) {
      xmit = radar->item;
      transmitter_loss = xmit->transmitter_loss;
      transmitter_power = xmit->transmitter_power * xmit->duty_cycle * pow(10.0, 0.1*xmit->pulse_ratio);
      fourpi_constant = 64*PI*PI*PI;
      range4 *= range4;
      transmitter_gain = xmit->antenna_table;
      prf = xmit->prf;
      pulse_width = xmit->pulse_width;
   }
      
   // set transmitter loss, power to nominal values
   else {
      transmitter_loss = 0.0;
      transmitter_gain = NULL;
      transmitter_power = 1.0;
      fourpi_constant = 16*PI*PI;
      prf = 0;
      pulse_width = 1;
   }

   receiver_loss = rec->receiver_loss;
   operating_loss = rec->operating_loss;
   receiver_gain = rec->antenna_table;
   receiver_thresholddb = rec->receiver_threshold;

   // calculate noise
   total_lossdb = receiver_loss + transmitter_loss + operating_loss;

   if (!(rec->flags & FLAG_ONE_M2)) {
      rxnoisedb = rec->receiver_noise;   
      return;
   }

   loss  = total_lossdb - receiver_thresholddb;
   gain = 0;

   if (transmitter_gain)
      gain += transmitter_gain->query_data(receiver_freq, 0, 0);

   if (receiver_gain)
      gain += receiver_gain->query_data(receiver_freq, 0,0);

   // note loss is negative, so we add/multiply instead of subtract/divide
   rxnoisedb = 10.0 * log10( (transmitter_power*wavelength2) /
               (fourpi_constant * range4 )) + (gain + loss);

   // IR
   if (sensor_type == SENSOR_IR) {
      presented_area_freq = rec->presented_area_freq;
      pixel_fov = rec->pixel_fov;
      noise_equiv_irradiance = rec->noise_equiv_irradiance_flag ? rec->noise_equiv_irradiance : rxnoisedb;

      // compute the signal-to-noise ratio
      rxnoisedb = 10.0 * log10(1.0 / (range4 * noise_equiv_irradiance));
   }

}


#ifdef asdf

asdf

gndrng - x,y range from receiver to target
rng - range from receiver to target
thresh - SENSING-THRESHOLD
xarea = BSRVsignature(SnrRcvr, PercElems);



BSRVoptical(atmos_atten, rng, gndrng, thresh, xarea);



static void BSRVoptical(
   double dBatten, // atmospheric attenuation
   double rng,     // distance from receiver to target
   double gndrng,  // 2D-distance from receiver to target
   double thresh,  // sensing threshold
   double xarea,   // tgt presented area (optical cross section)
   long *isee      // detection result (1=not detected, 5=detected)
) {
   // declare local variables
   long ii, numGlimpses, ngReacq;
   double gpSerch, gpTrack, gpReacq, cpSerch, cpTrack, cpReacq,
         bgndRadiance, pathRadiance, background, omega, contrast, atten,
         rate, factor, sensingPeriod;
   element *Element;              // (3)
   TelemInteracting *ElemInteracting;
   TTable *pTT, *GSerch, *GTrack, *GReacq;

   snr_rcvr_constant *SrxConst = TempSnrStore->SrxConst;
   snr_rcvr_data *SnrRcvrData = TempSnrStore->SnrRcvrData;
   optical_probs *OpticalProbs = BSRVstbufr->GetOpticalProbs();

   // retrieve sensor and target altitude values
   double snrAlt = RcvrPosV.Getz();
   double tgtAlt = TgtPosV.Getz();

   // look up background and path radiance
   pTT = TO_PTR2(TTable, SnrRcvrData->BackgndRadiance);
   bgndRadiance = pTT->GetValue(tgtAlt, snrAlt, gndrng);

   pTT = TO_PTR2(TTable, SnrRcvrData->PathRadiance);
   pathRadiance = pTT->GetValue(tgtAlt, snrAlt, gndrng);

   // compute background radiance at the sensor
   atten = DB_TO_NUM(atmos_atten);
   background = atten * bgndRadiance + pathRadiance;

   // compute the solid angle of the target at the sensor
   omega = xarea / (rng * rng);

   // retrieve the glimpse probability tables
   GSerch = TO_PTR2(TTable, SnrRcvrData->SerchGlimpse);
   GTrack = TO_PTR2(TTable, SnrRcvrData->TrackGlimpse);
   GReacq = TO_PTR2(TTable, SnrRcvrData->ReacqGlimpse);

   // when background radiance is greater than zero
   if (background > 1.e-6) {

      // loop over target elements with interactions with this sensor
      contrast = 0;
      for (ElemInteracting = BSRVfirstElem;
           ElemInteracting;
           ElemInteracting = ElemInteracting->getNext()) {

         // search for the maximum inherent contrast
         Element = ElemInteracting->getElement();
         if (Element->inherentContrast > contrast)
            contrast = Element->inherentContrast;
      }

      contrast *= atten * bgndRadiance / background;
   }

   // otherwise, zero background radiance
   else {
      // retrieve the highest contrast dimension value from glimpse tables
      contrast = GSerch->SearchRealPtr(omega)->GetMaxRange();
      contrast = OUR_MAX(contrast, GTrack->SearchRealPtr(omega)->GetMaxRange());
      contrast = OUR_MAX(contrast, GReacq->SearchRealPtr(omega)->GetMaxRange());
   }

   // look up the single glimpse probabilities
   gpSerch = GSerch->GetValue(omega, contrast);
   gpTrack = GTrack->GetValue(omega, contrast);
   gpReacq = GReacq->GetValue(omega, contrast);

   // determine the sensing period
   switch(BSRVstbufr->GetPrevMode()) {
      case SNR_OP_MODE_ACQ:
      case SNR_OP_MODE_SEARCH:   rate = SrxConst->acqfreq;   break;
      case SNR_OP_MODE_TRACK:    rate = SrxConst->trackfreq; break;
      case SNR_OP_MODE_GUIDANCE: rate = SrxConst->firefreq;  break;
      default: rate = 0; // for warning only
   }

   sensingPeriod = 1.0 / rate;

   // double period when results are already present and
   // sensing rates are not being strictly enforced
   if (BSRVstbufr->HasOldSensingResult() && !gpEnvironment->enforcedRates)
      sensingPeriod *= 2.0;

   // determine the number of glimpses in the sensing and reacquisition interval
   numGlimpses = OUR_MAX(1L, (long)(3.0 * sensingPeriod + 0.5));
   ngReacq = OUR_MAX(1L, (long)(3.0 * SrxConst->optReacqTime + 0.5));

   // retrieve the cumulative detection probabilities for this target
   cpSerch = OpticalProbs->cumSerchProb;
   cpTrack = OpticalProbs->cumTrackProb;
   cpReacq = OpticalProbs->cumReacqProb;
   
   // loop over the glimpses in the sensing interval
   factor = 1.0 / ngReacq;
   for (ii = numGlimpses; ii > 0; ii--) {

      // calculate probability of search, track, and reacquisition
      cpTrack = cpTrack * gpTrack + cpSerch * gpSerch + cpReacq * gpReacq;
      cpSerch = cpSerch * (1.0 - gpSerch) + (1.0 - gpReacq) * cpReacq * factor;
      cpReacq = 1.0 - cpTrack - cpSerch;
   }

   // store cumulative probabilities for next sensing chance
   OpticalProbs->cumSerchProb = cpSerch;
   OpticalProbs->cumTrackProb = cpTrack;
   OpticalProbs->cumReacqProb = cpReacq;

   // evaluate visibility for target
   return cpTrack >= thresh;
} // end BSRVoptical

#endif






/* **********************************************************************
   NOTE: rcs is from the emmitter table

   detection_data_type *data   // collection of radar calculation parameters
********************************************************************** */
double radar_type::calc_ir_detection_range(detection_data_type *data) {

   double range2;

   // compute the signal-to-noise ratio
   range2 = (data->rcs * pow(10.0, 0.1*(data->atmos_atten-rxnoisedb))) / noise_equiv_irradiance;

   return range2 > 1.0 ? sqrt(range2) : 1.0;
}


/* **********************************************************************
   This function calculates the maximum EM detection range for a given
   receiver/transmitter combo

   detection_data_type *data   // collection of radar calculation parameters
********************************************************************** */
double radar_type::calc_em_detection_range(detection_data_type *data) {

   double jammer_noise = 0;		//psnnnoi - to be calculated
   double mti_atten = 1.0;		// to be calculated
   double range4, gain;
   vector3d tempv;
   double temp;

   //********** CALCULATE GAIN ***********
   gain = 0;
   
   if (sensor_type == SENSOR_EM && transmitter_gain)
      gain += transmitter_gain->query_data(receiver_freq, data->beam_az, data->beam_el);

   if (receiver_gain)
      gain += receiver_gain->query_data(receiver_freq, data->beam_az, data->beam_el);

   //********** CALCULATE REST ***********

   range4 = (transmitter_power*wavelength2*pow(10.0, 0.1*(total_lossdb+gain+data->atmos_atten))*data->rcs*mti_atten) / 
            (fourpi_constant*pow(10.0, 0.1*receiver_thresholddb)*(pow(10.0, 0.1*rxnoisedb) + jammer_noise));

   if (range4 < 1)
      return 1;

   if (sensor_type == SENSOR_EM) {
      if (data->xmitter_pos) {     
         subeqarray3(tempv, data->xmitter_pos, data->target_pos);
         temp = dotproduct3(tempv, tempv);

         if (temp < 1)
            temp = 1;
      
         range4 = sqrt(range4 / temp);
         return range4 < 1 ? 1.0 : range4;
      }

      // asdf - temporary fix - no transmitter assumes receiver transmits...
      return pow(range4, 0.25);
   }

   // asdf - temporary fix - set transmitter dist to 1
   return sqrt(range4);      
}


/* **********************************************************************
   This function calculates the maximum detection range of a given sensor pair

   detection_data_type *data   // collection of radar calculation parameters
********************************************************************** */
double radar_type::calc_detection_range(detection_data_type *data) {

   switch (sensor_type) {
      case SENSOR_EM:
         return calc_em_detection_range(data);

      case SENSOR_IR:
         return calc_ir_detection_range(data);

      case SENSOR_OPTICAL:
//asdf - To be finished at a later date
return calc_em_detection_range(data);

      default:
         return 0.0;
   }
      
}


/* **********************************************************************
   This method calculates the maximum threashold range for a given target.
   
   - Assume main beam for gain - want potential threat dome
   - Assume receivers/xmitters are on and functional

   antenna_orientation_type *target_state, // XYZRPH of the target
   dbl_llist_manager *em_reflect,          // EM reflection table of target
   double *rec_pos,                        // XYZ of receiver
   double *xmit_pos,                       // XYZ of transmitter
   double *beam_az,                        // azimuth of target to receiver
   double *beam_el                         // elevation of target to receiver
********************************************************************** */
double radar_type::calc_detection_range(antenna_orientation_type *target_state, dbl_llist_manager *em_reflect, double *rec_pos, double *xmit_pos, double *beam_az, double *beam_el) {

   vector3d tempv;
   int rcs_flag;
   susceptibility_link *ttr;
   capability_list_type *ptr;
   double t;
   detection_data_type data;

   if (sensor_type == SENSOR_SONAR || !target_state)
      return -1;

   subeqarray3(tempv, target_state->pos, rec_pos);
   
   // target is at receiver!
   if (dotproduct3(tempv, tempv) < 1.0)
      return 1.0;

   //********** CALCULATE RCS/PresentArea ***********
   if (em_reflect) {
      calc_azel(target_state, rec_pos, sensor_type == SENSOR_EM && xmit_pos ? xmit_pos : NULL, &data.beam_az, &data.beam_el);

      for (data.rcs=0, rcs_flag = 0, ttr = (susceptibility_link *)em_reflect->head; ttr; ttr=(susceptibility_link *)ttr->next)
         if (ttr->list) {
            if (ttr->list->rlist) {
               data.rcs += ttr->list->rlist->query_data(receiver_freq, data.beam_az, data.beam_el);
               rcs_flag = 1;
            }

            if (ttr->list->elist)
               data.presented_area += ttr->list->elist->query_data(presented_area_freq, data.beam_az, data.beam_el);
         }

   }
   
   if (!rcs_flag)
      data.rcs = 1;	// 1 meter^2

   //********** CALCULATE ATMOS_ATTEN ***********

   for (ptr = (capability_list_type *)capability_man.head; ptr && !ptr->item->transmission_loss_table; ptr = (capability_list_type *)ptr->next);
 
   if (ptr) {
      subeqarray2(tempv, xmit_pos, rec_pos);
      t = dotproduct2(tempv, tempv);
      t = (t > 1.0f) ? sqrt(t) : 1.0;

      data.atmos_atten = ptr->item->transmission_loss_table->query_data(receiver_freq, xmit_pos[2], rec_pos[2], t);
   }

   copyarray3(data.target_pos, target_state->pos);
   copyarray3(data.xmitter_pos, xmit_pos);
   copyarray3(data.receiver_pos, rec_pos);

   return calc_detection_range(&data);
}


/* **********************************************************************
   This method calculates the maximum threashold range for a given target.
   
   - Assume main beam for gain - want potential threat dome
   - Assume receivers/xmitters are on and functional

   Platform *target                        // SWEG/JIMM target data structure
   dbl_llist_manager *em_reflect,          // EM reflection table of target
   Platform *receiver,                     // SWEG/JIMM receiver platform data structure
   Platform *xmitter,                      // SWEG/JIMM transmitter platform data structure
   double *beam_az,                        // azimuth of target to receiver
   double *beam_el                         // elevation of target to receiver
********************************************************************** */
/*
double radar_type::calc_detection_range(Platform *target, dbl_llist_manager *em_reflect, Platform *receiver, Platform *xmitter, double *beam_az, double *beam_el) {

   antenna_orientation_type target_state;
   vector3d rec_pos, xmit_pos;
   
   if (target) {
      target_state.pos[0] = get_float(target->x_coor);
      target_state.pos[1] = get_float(target->y_coor);
      target_state.pos[2] = get_float(target->z_coor);

      target_state.forward[0] = get_float(target->x_for_vec);
      target_state.forward[1] = get_float(target->y_for_vec);
      target_state.forward[2] = get_float(target->z_for_vec);

      target_state.up[0] = get_float(target->x_up_vec);
      target_state.up[1] = get_float(target->y_up_vec);
      target_state.up[2] = get_float(target->z_up_vec);

      rec_pos[0] = get_float(receiver->x_coor);
      rec_pos[1] = get_float(receiver->y_coor);
      rec_pos[2] = get_float(receiver->z_coor);

      if (sensor_type == SENSOR_EM && xmitter) {
         xmit_pos[0] = get_float(xmitter->x_coor);
         xmit_pos[1] = get_float(xmitter->y_coor);
         xmit_pos[2] = get_float(xmitter->z_coor);
         return calc_detection_range(&target_state, em_reflect, rec_pos, xmit_pos, beam_az, beam_el);
      }

      return calc_detection_range(&target_state, em_reflect, rec_pos, NULL, beam_az, beam_el);
   }

   return maxrange;
}
*/


/* ***********************************************************
   Dumps radar characteristics to stdio

   char *prefix  // tab delimited prefix
*********************************************************** */
void radar_type::dump_data(char *prefix) {

   capability_list_type *ctr;
   char buffer[1024];
      
   printf("%sRadar: %s %d ", prefix, name.string, id);

   if (semcode)
      printf("%d\n", semcode->semcode);
   else
      printf("Bad Semcode\n");
   
   sprintf(buffer, "%s      ", prefix);
   
   for (ctr=(capability_list_type *)capability_man.head; ctr; ctr=(capability_list_type *)ctr->next) {
      printf("%s   Capability:\t%s\t ", prefix, ctr->item->name.string);

      if (ctr->item->antenna_table) {
         printf("\n");
         ctr->item->antenna_table->dump_data(buffer);
      }
      
      else
         printf("*NO ANTENNA PATTERN!!!!*\n");      
   }
   
}


/* ***********************************************************
  Cleans up useless data nodes after parsing TDB
*********************************************************** */
void radar_type::data_cleanup() {

   capability_list_type *ptr, *qtr;
      
   // cleanup receivers
   ptr = (capability_list_type *)capability_man.head;
   
   while (ptr) {

      qtr = ptr;
      ptr = (capability_list_type *)ptr->next;

      if (!qtr->item->antenna_table) {
         capability_man.remove(qtr);
         delete qtr;
      }

   }

}


/* ***********************************************************
  Determines if enough data has been processed for this radar
  to be able to perform calculations
*********************************************************** */
int radar_type::query_justify() {

   return capability_man.head ? 1 : 0;
}

