

/* ***********************************************************************
                Naval Air Warfare Center / Aircraft Division              
                        Patuxent Naval Air Station                         
 ========================================================================

        File:          sw_platform.c                                       
                                                                        
        Classification: UNCLASSIFIED                                   
                                                                        
        Description:    This file contains the functions, methods, and
			global variables needed to manage SWEG platform
			in a running simulation.
			
        Written by:	Alex Harper
*********************************************************************** */


#include "vissweg.h"
#include "global.h"


/* ============================================================================
  This function performs dead recogning and position updates
 ============================================================================ */
void sweg_platform::update_data(double *machine_time) {

   double t;
  
   if (!plat)
      return;
 
   t = get_float(plat->game_time);

   if (plat_clone.timer >= t)
      return;

   plat_clone.init_pos[0] = plat_clone.pos[0] = get_float(plat->x_coor);      
   plat_clone.init_pos[1] = plat_clone.pos[1] = get_float(plat->y_coor);      
   plat_clone.init_pos[2] = plat_clone.pos[2] = get_float(plat->z_coor);      

   plat_clone.init_vel[0] = plat_clone.vel[0] = get_float(plat->x_vel);      
   plat_clone.init_vel[1] = plat_clone.vel[1] = get_float(plat->y_vel);      
   plat_clone.init_vel[2] = plat_clone.vel[2] = get_float(plat->z_vel);      

   plat_clone.acc[0] = get_float(plat->x_acc);      
   plat_clone.acc[1] = get_float(plat->y_acc);      
   plat_clone.acc[2] = get_float(plat->z_acc);      

   plat_clone.rpy[0] = -get_float(plat->roll);      
   plat_clone.rpy[1] = get_float(plat->pitch);      
   plat_clone.rpy[2] = get_float(plat->yaw) - HALFPI;      

   plat_clone.drpy[0] = -get_float(plat->roll_rate);      
   plat_clone.drpy[1] = get_float(plat->pitch_rate);      
   plat_clone.drpy[2] = get_float(plat->yaw_rate);

   plat_clone.dead_recogn_flag =
      dotproduct3(plat_clone.vel, plat_clone.vel) > CORRECT ||
      dotproduct3(plat_clone.acc, plat_clone.acc) > CORRECT ||
      dotproduct3(plat_clone.drpy, plat_clone.drpy) > CORRECT;

   plat_clone.timer = t;
   plat_clone.elapsed_time = 0;
   plat_clone.machine_time = *machine_time;

   platform_flag |= PLATFORM_FLAG_UPDATE | PLATFORM_FLAG_MOVEMENT;
}


/* ***************************************************************
*************************************************************** */
int sweg_platform::build_sensor_list(iface *isim) {

   int index, index2;
   System *system, *system2;
   sweg_sensor_type *sensor;
   platform_type *ptr;
   radar_type *radar;
   Semantic_Correlation *sctr;
   int scode, scode2;

   weaponman.dest();
   sensorman.dest();

   ptr = ((sweg_iface *)isim)->tdb_parser.find_platform(plat->plat_semantic_code);

   if (!ptr)
      return 0;

   for (index = plat->sys_lst; index; index = system->next_sys_from_plat) {
      system = (System *)&sweg[index];

      if ((system->sys_type/SYSTEM_TYPE) % 10 == SYSTEM_TYPE_SENSOR)
         switch((system->sys_type/SYSTEM_SUBTYPE) % 10) {
         
            case SYSTEM_SUBTYPE_RECEIVER:
               for (radar = (radar_type *)ptr->rec.head; radar && (!radar->semcode || system->semantic_code != radar->semcode->semcode); radar = (radar_type *)radar->next);

               if (!radar) {
                  sprintf(perror_buffer, "Warning: sensor receiver %d not found - possible disconnect between TDB and JIMM... Ignoring...\n", system->semantic_code);
                  pprintf(perror_buffer);
                  break;
               }

               sensorman.append(sensor = new sweg_sensor_type, NULL);
               sensor->sensor_rcvr = system;
               sensor->semcode_rcvr = system->semantic_code;
               sensor->sensor_data = radar;

               if (system->link) {
                  for (index2 = plat->sys_lst; index2; index2 = system2->next_sys_from_plat) {
                     system2 = (System *)&sweg[index2];
                     if (system2->glb_sys_id == system->link) {
                        sensor->sensor_xmit = system2;
                        sensor->semcode_xmit = system2->semantic_code;
                        break;
                     }

                  }

               }

               if (!sensor->sensor_xmit) {
                  sensor->sensor_xmit = sensor->sensor_rcvr;
                  sensor->semcode_xmit = sensor->semcode_rcvr;
               }

               sctr = ((sweg_iface *)isim)->find_correlation(sensor->semcode_xmit);

               if (sctr) {
                  scode = sctr->idcode;
                  scode2 = sctr->extracode;
               }

               else
                  scode = scode2 = 0;

               scode ? scode : scode2;

               sensor->str = (symbol_correlation_type *)symbol_lut.find(scode ? scode : scode2);
               sensor->id = sensor->sensor_xmit->glb_sys_id;
               break;

            default:
               break;
         }

   }

   return 1;
}


/* ***************************************************************
*************************************************************** */
int sweg_platform::update_sensors(iface *isim) {

   basic_radar_data_type radar_data;
   sweg_sensor_type *sstr;
   capability_list_type *radar;
   vector2d data, az_range, el_range;

   vis_data->perform(isim, PERFORM_PLATFORM_INIT_EMITTER, NULL);

   for (sstr = (sweg_sensor_type *)sensorman.head; sstr; sstr = (sweg_sensor_type *)sstr->next) {

      if (sstr->sensor_xmit->oper_stat != 1)
         continue;

      radar_data.id = sstr->id;
      radar_data.frequency = sstr->sensor_data->receiver_freq;
      radar_data.freq_range = 0;
      radar_data.power = sstr->sensor_data->transmitter_power;
      radar_data.prf = sstr->sensor_data->prf;
//      radar_data.pulse_width = sstr->sensor_data->pulse_width;
      radar_data.pulse_width = 299792458.0/radar_data.frequency;  //LIGHT_SPEED = 299792458.0  meters/sec in a vacuum

      switch (sstr->mode) {
         case SENSOR_MODE_ACQUIRE:
            radar_data.mode = RADAR_MODE_AQUISITION;
            break;

         case SENSOR_MODE_TRACKING:
            radar_data.mode = RADAR_MODE_TRACK;
            break;

         case SENSOR_MODE_GUIDANCE:
            radar_data.mode = RADAR_MODE_GUIDANCE;
            break;

//         case SENSOR_MODE_SEARCHING:
         default:
            radar_data.mode = RADAR_MODE_SEARCH;
            break;
      }
      
      // find xmitter through links
      if (sstr->sensor_data->links) {
         if (sstr->sensor_data->id == sstr->sensor_data->links->first)
            radar = (capability_list_type *)sstr->sensor_data->links->secondptr->capability_man.head;
         else 
            radar = (capability_list_type *)sstr->sensor_data->links->firstptr->capability_man.head;

         for (; radar && !(radar->item->typeflag & RADAR_TRANSMITTER); radar=(capability_list_type *)radar->next);

         if (radar && radar->item->antenna_table) {
            data[0] = az_range[0] = el_range[0] = 99999;
            data[1] = az_range[1] = el_range[1] = -99999;

            radar->item->antenna_table->query_minmax(data, az_range, el_range);

            radar_data.beam_az_sweep = 0.5 * (az_range[1] - az_range[0]);
            radar_data.beam_azimuth = az_range[0] + radar_data.beam_az_sweep;

            radar_data.beam_ele_sweep = 0.5 * (el_range[1] - el_range[0]);
            radar_data.beam_elevation = el_range[0] + radar_data.beam_ele_sweep;
         }

         else {
            radar_data.beam_az_sweep  = 0;
            radar_data.beam_azimuth   = 0;
            radar_data.beam_ele_sweep = 0;
            radar_data.beam_elevation = 0;
         }

      }

      else {
         radar_data.beam_az_sweep  = 0;
         radar_data.beam_azimuth   = 0;
         radar_data.beam_ele_sweep = 0;
         radar_data.beam_elevation = 0;
      }

      radar_data.str = sstr->str;

      vis_data->perform(isim, PERFORM_PLATFORM_EMITTER, &radar_data);
   }

   return 1;
}


/* ***************************************************************
  This method initializes this platform wrapper
*************************************************************** */
int sweg_platform::init(iface *isim) {
   
   symbol_correlation_type *str;
   correlate *basis;

   id = plat->glb_SWEG_plat_id;

   build_sensor_list(isim);

   str=(symbol_correlation_type *)symbol_lut.find(type);

   if (!str) {
      sprintf(perror_buffer, "WARNING: Unable to identify entity # %d\n", type);
      pprintf(perror_buffer);
      return 0;
   }
         
   // find type
   if (!isim->platman.head) {
      pprintf("No correlation table\n");
      return 0;
   }

   basis = ((correlate *)isim->platman.head)->find(str->sym_3D);

   if (!basis) {
      sprintf(perror_buffer, "Warning:Bad Correlation Code %d\n", str->sym_3D);
      pprintf(perror_buffer);
      return 0;
   }

   if (((str->sym_3D/1000) % 100) == 2 || ((str->sym_3D/100000000) % 100) == 2)
      platform_flag |= PLATFORM_FLAG_FLY;
   
   // create rendering object
   if (vis_data) {
      delete vis_data;
      vis_data = NULL;
   }

   vis_data = ((platform_data *(*)(basic_platform *))isim->data_loader)(this);

   if (!vis_data)
      return 0;
  
   if (!vis_data->init(isim, basis, str)) {
      delete vis_data;
      vis_data = NULL;
      return 0;
   }

   return 1;
}


/* **************************************************************
************************************************************** */
int sweg_platform::perform(iface *isim, int task, void *data) {

   platform_type *pttr;
   radar_type *rtr;
   capability_list_type *ctr;
   range_type *rangetr;

   switch (task) {
/*   
      case PERFORM_QUERY_SYMBOL_CODE:
         *(int *)data = type_code;
         return 1;
*/

      case PERFORM_QUERY_MAX_RADAR_RANGE:

         *(double *)data = 0;

         if (!(platform_flag & (PLATFORM_FLAG_CONTROL | PLATFORM_FLAG_EXTERNAL)))
            return 0;
	   
         pttr = ((sweg_iface *)isim)->tdb_parser.find_platform(plat->plat_semantic_code);

         if (!pttr)
            return 0;
	    
         for (rtr = (radar_type *)pttr->rec.head; rtr; rtr = (radar_type *)rtr->next)
            for (ctr = (capability_list_type *)rtr->capability_man.head; ctr; ctr = (capability_list_type *)ctr->next) {
               rangetr = (range_type *)ctr->item->range_list.tail;
               if (rangetr && *(double *)data < rangetr->range)
                  *(double *)data = rangetr->range;
            }

         return 1;

      case PERFORM_UPDATE_SENSORS:
         update_sensors(isim);
         break;

      default:
         break;
   }
      
   return basic_platform::perform(isim, task, data);
}


/* ***************************************************************************
*************************************************************************** */
void sweg_platform::update_platform(plat_stat_type *pc) {

   int index;
   Scenario *scen;
   
   plat_clone = *pc;
   
   // a guess - to say I'm writing
   plat->acc_flags = -1;

   plat->x_coor.f = (float)plat_clone.pos[0];
   plat->y_coor.f = (float)plat_clone.pos[1];
   plat->z_coor.f = (float)plat_clone.pos[2]; 

   plat->x_vel.f = (float)plat_clone.vel[0];
   plat->y_vel.f = (float)plat_clone.vel[1];
   plat->z_vel.f = (float)plat_clone.vel[2];
   
   plat->x_acc.f = (float)plat_clone.acc[0];
   plat->y_acc.f = (float)plat_clone.acc[1];
   plat->z_acc.f = (float)plat_clone.acc[2]; 

   plat->pitch.f = (float)plat_clone.rpy[1];
   plat->yaw.f = plat->heading.f = (float)(plat_clone.rpy[2] + HALFPI);
   plat->roll.f  = (float)-plat_clone.rpy[0];

   plat->pitch_rate.f = (float)plat_clone.drpy[1];
   plat->yaw_rate.f   = (float)plat_clone.drpy[2];
   plat->roll_rate.f  = (float)-plat_clone.drpy[0];

   index = (get_master())->scen_pars;
   scen = (Scenario *)&sweg[index];

   // another guess
   plat_clone.timer = plat->game_time.f = get_float(scen->game_time);

   plat->upd_flags = 1;
   plat->acc_flags = 0;
   platform_flag |= PLATFORM_FLAG_UPDATE;
}


